@@ -15,7 +15,7 @@ import json
 import argparse
 
 from abc import ABC
-from enum import IntEnum
+from enum import IntEnum, auto, unique
 from bcc import BPF
 from prettytable import PrettyTable
 
@@ -63,11 +63,12 @@ class TraceEvents(IntEnum):
     Events to trace
     """
 
-    TRANSACTION = 1
-    QUERY = 2
-    TABLE = 3
-    LOCK = 4
-    ERROR = 5
+    TRANSACTION = auto()
+    QUERY = auto()
+    TABLE = auto()
+    LOCK = auto()
+    INVALIDATION = auto()
+    ERROR = auto()
 
 
 parser = argparse.ArgumentParser(
@@ -155,10 +156,13 @@ parser.add_argument(
 )
 
 
+@unique
 class Events(IntEnum):
-    TABLE_OPEN = 0
-    TABLE_CLOSE = 1
-    ERROR = 2
+    TABLE_OPEN = 1
+    TABLE_OPEN_RV = 2
+    TABLE_OPEN_RV_EXTENDED = 3
+    TABLE_CLOSE = 4
+    ERROR = 5
     QUERY_BEGIN = 20
     QUERY_END = 21
     LOCK_RELATION_OID = 30
@@ -173,12 +177,14 @@ class Events(IntEnum):
     TRANSACTION_BEGIN = 40
     TRANSACTION_COMMIT = 41
     TRANSACTION_ABORT = 42
+    INVALIDATION_MESSAGES_ACCEPT = 50
     # Events over 1000 are handled regardless of any pid filter
     GLOBAL = 1000
     DEADLOCK = 1001
 
 
 # From elog.h
+@unique
 class PGError(IntEnum):
     ERROR = 21
     FATAL = 22
@@ -187,7 +193,6 @@ class PGError(IntEnum):
 
 class LockStatisticsEntry:
     def __init__(self) -> None:
-
         # The number of requested locks
         self._lock_count = 0
 
@@ -258,7 +263,6 @@ class PGLockTraceOutput(ABC):
         Add the lock call to the statistics and measure lock request time
         """
         if event.event_type == Events.LOCK_RELATION_OID:
-
             if oid_value not in self.statistics:
                 self.statistics[oid_value] = LockStatisticsEntry()
 
@@ -341,7 +345,6 @@ class PGLockTraceOutput(ABC):
 
 
 class PGLockTraceOutputHuman(PGLockTraceOutput):
-
     # pylint: disable=too-many-branches, too-many-statements
     def print_event(self, _cpu, data, _size):
         """
@@ -371,6 +374,15 @@ class PGLockTraceOutputHuman(PGLockTraceOutput):
         if event.event_type == Events.TABLE_OPEN:
             lock_type = PostgreSQLLockHelper.lock_type_to_str(event.mode)
             output = f"{print_prefix} Table open {tablename} {lock_type}"
+        elif event.event_type in (Events.TABLE_OPEN_RV, Events.TABLE_OPEN_RV_EXTENDED):
+            # Table is opened using a (string) range value
+            lock_type = PostgreSQLLockHelper.lock_type_to_str(event.mode)
+            schema = event.payload_str1.decode("utf-8")
+            table = event.payload_str2.decode("utf-8")
+            tablename = f"{schema}.{table}"
+            output = (
+                f"{print_prefix} Table open (by range value) {tablename} {lock_type}"
+            )
         elif event.event_type == Events.TABLE_CLOSE:
             lock_type = PostgreSQLLockHelper.lock_type_to_str(event.mode)
             output = f"{print_prefix} Table close {tablename} {lock_type}"
@@ -413,11 +425,13 @@ class PGLockTraceOutputHuman(PGLockTraceOutput):
                 f"{print_prefix} Lock ungranted (local) {tablename} {lock_type} "
                 f"(Hold local {event.lock_local_hold})"
             )
+        elif event.event_type == Events.INVALIDATION_MESSAGES_ACCEPT:
+            output = f"{print_prefix} Accept invalidation messages"
         elif event.event_type == Events.ERROR:
             pgerror_value = PGError(event.mode).name
             output = f"{print_prefix} Error occurred servity: {pgerror_value}"
         elif event.event_type == Events.QUERY_BEGIN:
-            query = event.payload.decode("utf-8")
+            query = event.payload_str1.decode("utf-8")
             output = f"{print_prefix} Query begin '{query}'"
         elif event.event_type == Events.QUERY_END:
             output = f"{print_prefix} Query done\n"
@@ -430,7 +444,7 @@ class PGLockTraceOutputHuman(PGLockTraceOutput):
         elif event.event_type == Events.DEADLOCK:
             output = f"{print_prefix} DEADLOCK DETECTED"
         else:
-            raise Exception(f"Unsupported event type {event.event_type}")
+            raise ValueError(f"Unsupported event type {event.event_type}")
 
         self.handle_output_line(output)
         self.print_stacktace_if_available(event)
@@ -473,6 +487,8 @@ class PGLockTraceOutputJSON(PGLockTraceOutput):
 
         if event.event_type in (
             Events.TABLE_OPEN,
+            Events.TABLE_OPEN_RV,
+            Events.TABLE_OPEN_RV_EXTENDED,
             Events.TABLE_CLOSE,
             Events.LOCK_RELATION_OID,
             Events.UNLOCK_RELATION_OID,
@@ -502,7 +518,12 @@ class PGLockTraceOutputJSON(PGLockTraceOutput):
             pgerror_value = PGError(event.mode).name
             output["servity"] = pgerror_value
         elif event.event_type == Events.QUERY_BEGIN:
-            output["query"] = event.payload.decode("utf-8")
+            output["query"] = event.payload_str1.decode("utf-8")
+        elif event.event_type in (Events.TABLE_OPEN_RV, Events.TABLE_OPEN_RV_EXTENDED):
+            # Table is opened using a (string) range value
+            schema = event.payload_str1.decode("utf-8")
+            table = event.payload_str2.decode("utf-8")
+            output["table"] = f"{schema}.{table}"
         elif event.event_type == Events.LOCK_GRANTED_LOCAL:
             output["lock_local_hold"] = event.lock_local_hold
         elif event.event_type == Events.LOCK_RELATION_OID_END:
@@ -550,9 +571,8 @@ class PGLockTracer:
         self.oid_resolvers = {}
 
         for oid_resolver_url in self.args.oid_resolver_urls:
-
             if ":" not in oid_resolver_url:
-                raise Exception(
+                raise ValueError(
                     f"Resolver URL has to be in format: 'PID:URL' ({oid_resolver_url} was provided)"
                 )
 
@@ -574,22 +594,21 @@ class PGLockTracer:
 
         # Belong the processes to the binary?
         for pid in self.args.pids:
-
             if not os.path.isdir(f"/proc/{pid}"):
-                raise Exception(
+                raise ValueError(
                     f"/proc entry for pid {pid} not found, does the process exist?"
                 )
 
             binary = os.readlink(f"/proc/{pid}/exe")
 
             if binary != self.args.path:
-                raise Exception(
+                raise ValueError(
                     f"Pid {pid} does not belong to binary {self.args.path}. Executable is {binary}"
                 )
 
         # Does the output file already exists?
         if self.args.output_file and os.path.exists(self.args.output_file):
-            raise Exception(f"Output file {self.args.output_file} already exists")
+            raise ValueError(f"Output file {self.args.output_file} already exists")
 
     @staticmethod
     def generate_c_defines(stacktrace_events, verbose):
@@ -653,7 +672,9 @@ class PGLockTracer:
             # pylint: disable=consider-using-with
             self.output_file = open(self.args.output_file, "w", encoding="utf-8")
             if not self.output_file.writable():
-                raise Exception(f"Output file {self.args.output_file} is not writeable")
+                raise ValueError(
+                    f"Output file {self.args.output_file} is not writeable"
+                )
 
         # Output as human readable text or as json?
         self.output_class = (
@@ -684,7 +705,7 @@ class PGLockTracer:
         )
 
         if not func_and_addr:
-            raise Exception(f"Unable to locate function {function_regex}")
+            raise ValueError(f"Unable to locate function {function_regex}")
 
         # Handle address duplicates
         for function, address in func_and_addr:
@@ -725,6 +746,8 @@ class PGLockTracer:
         # Table probes
         if self.args.trace is None or TraceEvents.TABLE.name in self.args.trace:
             self.register_probe("^table_open$", "bpf_table_open")
+            self.register_probe("^table_openrv$", "bpf_table_openrv")
+            self.register_probe("^table_openrv_extended$", "bpf_table_openrv_extended")
             self.register_probe("^table_close$", "bpf_table_close")
 
         # Lock probes
@@ -743,7 +766,13 @@ class PGLockTracer:
                 "^FastPathUnGrantRelationLock$",
                 "bpf_lock_fastpath_ungrant",
             )
-            self.register_probe("^RemoveLocalLock$", "bfp_local_local_ungrant")
+            self.register_probe("^RemoveLocalLock$", "bfp_local_lock_ungrant")
+
+        # Invalidation messages probes
+        if self.args.trace is None or TraceEvents.INVALIDATION.name in self.args.trace:
+            self.register_probe(
+                "^AcceptInvalidationMessages$", "bpf_accept_invalidation_messages"
+            )
 
         # Error probes
         if self.args.trace is None or TraceEvents.ERROR.name in self.args.trace:
